'use strict';

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const AWS = require('aws-sdk');

function testStage(stage, deploymentDefinitionStage) {
  let regexp;

  if (/^\/.*\/(i|g|m|s|u|y)*$/.test(deploymentDefinitionStage)) {
    const pattern = deploymentDefinitionStage.substr(1, deploymentDefinitionStage.lastIndexOf('/') - 1);
    const flag = deploymentDefinitionStage.substr(deploymentDefinitionStage.lastIndexOf('/') + 1);
    regexp = new RegExp(pattern, flag);
  } else {
    regexp = new RegExp(`^${deploymentDefinitionStage}$`);
  }

  return regexp.test(stage);
}

class DeploymentManagerPlugin {
  constructor(serverless, options) {
    this.serverless = serverless;
    this.options = options;
    this.commands = {
      validate: {
        usage: 'Checks that stage matched current AWS account id defined in serverless.yml',
        lifecycleEvents: ['validate'],
        options: {
          stage: {
            usage: ' "--stage dev" or "-s dev"',
            required: true,
            shortcut: 's'
          }
        }
      }
    };
    this.hooks = {
      'validate:validate': this.validateStageAndAccount.bind(this),
      'before:package:initialize': this.validateStageAndAccount.bind(this),
      'before:deploy:function:initialize': this.validateStageAndAccount.bind(this)
    };
  }

  validateStageAndAccount() {
    var _this = this;

    return _asyncToGenerator(function* () {
      const {
        processedInput,
        service
      } = _this.serverless;

      if (!service.custom.deployment) {
        throw new Error('[serverless-deployment-guard] service.custom.deployment definition is missing');
      }

      const deploymentDefinitions = service.custom.deployment.filter(({
        stage
      }) => testStage(processedInput.options.stage, stage));

      if (deploymentDefinitions.length === 0) {
        throw new Error(`[serverless-deployment-guard] stage '${processedInput.options.stage}' cannot be deployed`);
      }

      if (deploymentDefinitions.every(item => !!item.accountId)) {
        const sts = new AWS.STS({
          region: processedInput.options.region
        });
        const {
          Account
        } = yield sts.getCallerIdentity().promise();
        const exists = !!service.custom.deployment.find(item => {
          return item.stage && item.accountId && testStage(processedInput.options.stage, item.stage) && item.accountId.toString() === Account;
        });

        if (!exists) {
          throw new Error(`[serverless-deployment-guard] stage '${processedInput.options.stage}' cannot be deployed to account '${Account}'`);
        }
      }
    })();
  }

}

module.exports = DeploymentManagerPlugin;